@@ -5,8 +5,8 @@
  | program/include/rcmail.php                                            |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2013, The Roundcube Dev Team                       |
- | Copyright (C) 2011-2013, Kolab Systems AG                             |
+ | Copyright (C) 2008-2014, The Roundcube Dev Team                       |
+ | Copyright (C) 2011-2014, Kolab Systems AG                             |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -25,7 +25,7 @@
  * Application class of Roundcube Webmail
  * implemented as singleton
  *
- * @package Core
+ * @package Webmail
  */
 class rcmail extends rcube
 {
@@ -106,21 +106,23 @@ class rcmail extends rcube
         // reset some session parameters when changing task
         if ($this->task != 'utils') {
             // we reset list page when switching to another task
-            // but only to the main task interface - empty action (#1489076)
+            // but only to the main task interface - empty action (#1489076, #1490116)
             // this will prevent from unintentional page reset on cross-task requests
             if ($this->session && $_SESSION['task'] != $this->task && empty($this->action)) {
                 $this->session->remove('page');
-            }
 
-            // set current task to session
-            $_SESSION['task'] = $this->task;
+                // set current task to session
+                $_SESSION['task'] = $this->task;
+            }
         }
 
-        // init output class
-        if (!empty($_REQUEST['_remote']))
+        // init output class (not in CLI mode)
+        if (!empty($_REQUEST['_remote'])) {
             $GLOBALS['OUTPUT'] = $this->json_init();
-        else
+        }
+        else if ($_SERVER['REMOTE_ADDR']) {
             $GLOBALS['OUTPUT'] = $this->load_gui(!empty($_REQUEST['_framed']));
+        }
 
         // load plugins
         $this->plugins->init($this, $this->task);
@@ -139,6 +141,8 @@ class rcmail extends rcube
 
         if ($this->user && $this->user->ID)
             $task = !$task ? 'mail' : $task;
+        else if (php_sapi_name() == 'cli')
+            $task = 'cli';
         else
             $task = 'login';
 
@@ -162,12 +166,7 @@ class rcmail extends rcube
      */
     public function set_user($user)
     {
-        if (is_object($user)) {
-            $this->user = $user;
-
-            // overwrite config with user preferences
-            $this->config->set_user_prefs((array)$this->user->get_prefs());
-        }
+        parent::set_user($user);
 
         $lang = $this->language_prop($this->config->get('language', $_SESSION['language']));
         $_SESSION['language'] = $this->user->language = $lang;
@@ -175,9 +174,11 @@ class rcmail extends rcube
         // set localization
         setlocale(LC_ALL, $lang . '.utf8', $lang . '.UTF-8', 'en_US.utf8', 'en_US.UTF-8');
 
-        // workaround for http://bugs.php.net/bug.php?id=18556
-        if (version_compare(PHP_VERSION, '5.5.0', '<') && in_array($lang, array('tr_TR', 'ku', 'az_AZ'))) {
-            setlocale(LC_CTYPE, 'en_US.utf8', 'en_US.UTF-8');
+        // Workaround for http://bugs.php.net/bug.php?id=18556
+        // Also strtoupper/strtolower and other methods are locale-aware
+        // for these locales it is problematic (#1490519)
+        if (in_array($lang, array('tr_TR', 'ku', 'az_AZ'))) {
+            setlocale(LC_CTYPE, 'en_US.utf8', 'en_US.UTF-8', 'C');
         }
     }
 
@@ -439,7 +440,8 @@ class rcmail extends rcube
         $this->output->set_env('compose_extwin', $this->config->get('compose_extwin',false));
 
         // add some basic labels to client
-        $this->output->add_label('loading', 'servererror', 'connerror', 'requesttimedout', 'refreshing');
+        $this->output->add_label('loading', 'servererror', 'connerror', 'requesttimedout',
+            'refreshing', 'windowopenerror', 'uploadingmany');
 
         return $this->output;
     }
@@ -590,6 +592,8 @@ class rcmail extends rcube
 
         // try to log in
         if (!$storage->connect($host, $username, $pass, $port, $ssl)) {
+            // Wait a second to slow down brute-force attacks (#1490549)
+            sleep(1);
             return false;
         }
 
@@ -645,10 +649,8 @@ class rcmail extends rcube
             // fix some old settings according to namespace prefix
             $this->fix_namespace_settings($user);
 
-            // create default folders on login
-            if ($this->config->get('create_default_folders')) {
-                $storage->create_default_folders();
-            }
+            // set/create special folders
+            $this->set_special_folders();
 
             // clear all mailboxes related cache(s)
             $storage->clear_cache('mailboxes', true);
@@ -742,14 +744,16 @@ class rcmail extends rcube
      */
     public function logout_actions()
     {
-        $config  = $this->config->all();
-        $storage = $this->get_storage();
+        $storage        = $this->get_storage();
+        $logout_expunge = $this->config->get('logout_expunge');
+        $logout_purge   = $this->config->get('logout_purge');
+        $trash_mbox     = $this->config->get('trash_mbox');
 
-        if ($config['logout_purge'] && !empty($config['trash_mbox'])) {
-            $storage->clear_folder($config['trash_mbox']);
+        if ($logout_purge && !empty($trash_mbox)) {
+            $storage->clear_folder($trash_mbox);
         }
 
-        if ($config['logout_expunge']) {
+        if ($logout_expunge) {
             $storage->expunge_folder('INBOX');
         }
 
@@ -759,48 +763,17 @@ class rcmail extends rcube
         }
     }
 
-    /**
-     * Generate a unique token to be used in a form request
-     *
-     * @return string The request token
-     */
-    public function get_request_token()
-    {
-        $sess_id = $_COOKIE[ini_get('session.name')];
-
-        if (!$sess_id) {
-            $sess_id = session_id();
-        }
-
-        $plugin = $this->plugins->exec_hook('request_token', array(
-            'value' => md5('RT' . $this->get_user_id() . $this->config->get('des_key') . $sess_id)));
-
-        return $plugin['value'];
-    }
-
-    /**
-     * Check if the current request contains a valid token
-     *
-     * @param int Request method
-     *
-     * @return boolean True if request token is valid false if not
-     */
-    public function check_request($mode = rcube_utils::INPUT_POST)
-    {
-        $token   = rcube_utils::get_input_value('_token', $mode);
-        $sess_id = $_COOKIE[ini_get('session.name')];
-
-        return !empty($sess_id) && $token == $this->get_request_token();
-    }
-
     /**
      * Build a valid URL to this instance of Roundcube
      *
-     * @param mixed Either a string with the action or url parameters as key-value pairs
+     * @param mixed   Either a string with the action or url parameters as key-value pairs
+     * @param boolean Build an URL absolute to document root
+     * @param boolean Create fully qualified URL including http(s):// and hostname
+     * @param bool    Return absolute URL in secure location
      *
      * @return string Valid application URL
      */
-    public function url($p)
+    public function url($p, $absolute = false, $full = false, $secure = false)
     {
         if (!is_array($p)) {
             if (strpos($p, 'http') === 0) {
@@ -810,14 +783,15 @@ class rcmail extends rcube
             $p = array('_action' => @func_get_arg(0));
         }
 
-        $task = $p['_task'] ? $p['_task'] : ($p['task'] ? $p['task'] : $this->task);
-        $p['_task'] = $task;
-        unset($p['task']);
+        $pre = array();
+        $task = $p['_task'] ?: ($p['task'] ?: $this->task);
+        $pre['_task'] = $task;
+        unset($p['task'], $p['_task']);
 
-        $url  = './' . $this->filename;
+        $url  = $this->filename;
         $delm = '?';
 
-        foreach (array_reverse($p) as $key => $val) {
+        foreach (array_merge($pre, $p) as $key => $val) {
             if ($val !== '' && $val !== null) {
                 $par  = $key[0] == '_' ? $key : '_'.$key;
                 $url .= $delm.urlencode($par).'='.urlencode($val);
@@ -825,7 +799,40 @@ class rcmail extends rcube
             }
         }
 
-        return $url;
+        $base_path = strval($_SERVER['REDIRECT_SCRIPT_URL'] ?: $_SERVER['SCRIPT_NAME']);
+        $base_path = preg_replace('![^/]+$!', '', $base_path);
+
+        if ($secure && ($token = $this->get_secure_url_token(true))) {
+            // add token to the url
+            $url = $token . '/' . $url;
+
+            // remove old token from the path
+            $base_path = rtrim($base_path, '/');
+            $base_path = preg_replace('/\/[a-f0-9]{' . strlen($token) . '}$/', '', $base_path);
+
+            // this need to be full url to make redirects work
+            $absolute = true;
+        }
+        else if ($secure && ($token = $this->get_request_token()))
+            $url .= $delm . '_token=' . urlencode($token);
+
+        if ($absolute || $full) {
+            // add base path to this Roundcube installation
+            if ($base_path == '') $base_path = '/';
+            $prefix = $base_path;
+
+            // prepend protocol://hostname:port
+            if ($full) {
+                $prefix = rcube_utils::resolve_url($prefix);
+            }
+
+            $prefix = rtrim($prefix, '/') . '/';
+        }
+        else {
+            $prefix = './';
+        }
+
+        return $prefix . $url;
     }
 
     /**
@@ -841,7 +848,7 @@ class rcmail extends rcube
         }
 
         // write performance stats to logs/console
-        if ($this->config->get('devel_mode')) {
+        if ($this->config->get('devel_mode') || $this->config->get('performance_stats')) {
             // make sure logged numbers use unified format
             setlocale(LC_NUMERIC, 'en_US.utf8', 'en_US.UTF-8', 'en_US', 'C');
 
@@ -859,6 +866,28 @@ class rcmail extends rcube
         }
     }
 
+    /**
+     * CSRF attack prevention code
+     *
+     * @param int Request mode
+     */
+    public function request_security_check($mode = rcube_utils::INPUT_POST)
+    {
+        // check request token
+        if (!$this->check_request($mode)) {
+            self::raise_error(array(
+                'code' => 403, 'type' => 'php',
+                'message' => "Request security check failed"), false, true);
+        }
+
+        // check referer if configured
+        if ($this->config->get('referer_check') && !rcube_utils::check_referer()) {
+            self::raise_error(array(
+                'code' => 403, 'type' => 'php',
+                'message' => "Referer check failed"), true, true);
+        }
+    }
+
     /**
      * Registers action aliases for current task
      *
@@ -899,12 +928,15 @@ class rcmail extends rcube
         $prefix     = $this->storage->get_namespace('prefix');
         $prefix_len = strlen($prefix);
 
-        if (!$prefix_len)
+        if (!$prefix_len) {
             return;
+        }
 
-        $prefs = $this->config->all();
-        if (!empty($prefs['namespace_fixed']))
+        if ($this->config->get('namespace_fixed')) {
             return;
+        }
+
+        $prefs = array();
 
         // Build namespace prefix regexp
         $ns     = $this->storage->get_namespace();
@@ -924,24 +956,16 @@ class rcmail extends rcube
         // Fix preferences
         $opts = array('drafts_mbox', 'junk_mbox', 'sent_mbox', 'trash_mbox', 'archive_mbox');
         foreach ($opts as $opt) {
-            if ($value = $prefs[$opt]) {
+            if ($value = $this->config->get($opt)) {
                 if ($value != 'INBOX' && !preg_match($regexp, $value)) {
                     $prefs[$opt] = $prefix.$value;
                 }
             }
         }
 
-        if (!empty($prefs['default_folders'])) {
-            foreach ($prefs['default_folders'] as $idx => $name) {
-                if ($name != 'INBOX' && !preg_match($regexp, $name)) {
-                    $prefs['default_folders'][$idx] = $prefix.$name;
-                }
-            }
-        }
-
-        if (!empty($prefs['search_mods'])) {
+        if (($search_mods = $this->config->get('search_mods')) && !empty($search_mods)) {
             $folders = array();
-            foreach ($prefs['search_mods'] as $idx => $value) {
+            foreach ($search_mods as $idx => $value) {
                 if ($idx != 'INBOX' && $idx != '*' && !preg_match($regexp, $idx)) {
                     $idx = $prefix.$idx;
                 }
@@ -951,9 +975,9 @@ class rcmail extends rcube
             $prefs['search_mods'] = $folders;
         }
 
-        if (!empty($prefs['message_threading'])) {
+        if (($threading = $this->config->get('message_threading')) && !empty($threading)) {
             $folders = array();
-            foreach ($prefs['message_threading'] as $idx => $value) {
+            foreach ($threading as $idx => $value) {
                 if ($idx != 'INBOX' && !preg_match($regexp, $idx)) {
                     $idx = $prefix.$idx;
                 }
@@ -963,8 +987,8 @@ class rcmail extends rcube
             $prefs['message_threading'] = $folders;
         }
 
-        if (!empty($prefs['collapsed_folders'])) {
-            $folders     = explode('&&', $prefs['collapsed_folders']);
+        if ($collapsed = $this->config->get('collapsed_folders')) {
+            $folders     = explode('&&', $collapsed);
             $count       = count($folders);
             $folders_str = '';
 
@@ -1097,14 +1121,17 @@ class rcmail extends rcube
         }
         else {
             foreach ($table_data as $row_data) {
-                $class = !empty($row_data['class']) ? $row_data['class'] : '';
+                $class = !empty($row_data['class']) ? $row_data['class'] : null;
+                if (!empty($attrib['rowclass']))
+                    $class = trim($class . ' ' . $attrib['rowclass']);
                 $rowid = 'rcmrow' . rcube_utils::html_identifier($row_data[$id_col]);
 
                 $table->add_row(array('id' => $rowid, 'class' => $class));
 
                 // format each col
                 foreach ($a_show_cols as $col) {
-                    $table->add($col, $this->Q(is_array($row_data[$col]) ? $row_data[$col][0] : $row_data[$col]));
+                    $val = is_array($row_data[$col]) ? $row_data[$col][0] : $row_data[$col];
+                    $table->add($col, empty($attrib['ishtml']) ? $this->Q($val) : $val);
                 }
             }
         }
@@ -1166,11 +1193,11 @@ class rcmail extends rcube
             $week_limit  = mktime(0, 0, 0, $now_date['mon'], $now_date['mday']-6, $now_date['year']);
             $pretty_date = $this->config->get('prettydate');
 
-            if ($pretty_date && $timestamp > $today_limit && $timestamp < $now) {
+            if ($pretty_date && $timestamp > $today_limit && $timestamp <= $now) {
                 $format = $this->config->get('date_today', $this->config->get('time_format', 'H:i'));
                 $today  = true;
             }
-            else if ($pretty_date && $timestamp > $week_limit && $timestamp < $now) {
+            else if ($pretty_date && $timestamp > $week_limit && $timestamp <= $now) {
                 $format = $this->config->get('date_short', 'D H:i');
             }
             else {
@@ -1511,7 +1538,7 @@ class rcmail extends rcube
             $html_name = $this->Q($foldername) . ($unread ? html::span('unreadcount', sprintf($attrib['unreadwrap'], $unread)) : '');
             $link_attrib = $folder['virtual'] ? array() : array(
                 'href' => $this->url(array('_mbox' => $folder['id'])),
-                'onclick' => sprintf("return %s.command('list','%s',this)", rcmail_output::JS_OBJECT_NAME, $js_name),
+                'onclick' => sprintf("return %s.command('list','%s',this,event)", rcmail_output::JS_OBJECT_NAME, $js_name),
                 'rel' => $folder['id'],
                 'title' => $title,
             );
@@ -1563,7 +1590,7 @@ class rcmail extends rcube
             // skip folders in which it isn't possible to create subfolders
             if (!empty($opts['skip_noinferiors'])) {
                 $attrs = $this->storage->folder_attributes($folder['id']);
-                if ($attrs && in_array('\\Noinferiors', $attrs)) {
+                if ($attrs && in_array_nocase('\\Noinferiors', $attrs)) {
                     continue;
                 }
             }
@@ -1651,14 +1678,13 @@ class rcmail extends rcube
     public function localize_folderpath($path)
     {
         $protect_folders = $this->config->get('protect_default_folders');
-        $default_folders = (array) $this->config->get('default_folders');
         $delimiter       = $this->storage->get_hierarchy_delimiter();
         $path            = explode($delimiter, $path);
         $result          = array();
 
         foreach ($path as $idx => $dir) {
             $directory = implode($delimiter, array_slice($path, 0, $idx+1));
-            if ($protect_folders && in_array($directory, $default_folders)) {
+            if ($protect_folders && $this->storage->is_special_folder($directory)) {
                 unset($result);
                 $result[] = $this->localize_foldername($directory);
             }
@@ -1691,13 +1717,14 @@ class rcmail extends rcube
     }
 
 
-    public function quota_content($attrib = null)
+    public function quota_content($attrib = null, $folder = null)
     {
-        $quota = $this->storage->get_quota();
+        $quota = $this->storage->get_quota($folder);
         $quota = $this->plugins->exec_hook('quota', $quota);
 
         $quota_result = (array) $quota;
-        $quota_result['type'] = isset($_SESSION['quota_display']) ? $_SESSION['quota_display'] : '';
+        $quota_result['type']   = isset($_SESSION['quota_display']) ? $_SESSION['quota_display'] : '';
+        $quota_result['folder'] = $folder !== null && $folder !== '' ? $folder : 'INBOX';
 
         if ($quota['total'] > 0) {
             if (!isset($quota['percent'])) {
@@ -1714,7 +1741,39 @@ class rcmail extends rcube
                 $quota_result['width'] = $attrib['width'];
             }
             if ($attrib['height']) {
-                $quota_result['height']	= $attrib['height'];
+                $quota_result['height'] = $attrib['height'];
+            }
+
+            // build a table of quota types/roots info
+            if (($root_cnt = count($quota_result['all'])) > 1 || count($quota_result['all'][key($quota_result['all'])]) > 1) {
+                $table = new html_table(array('cols' => 3, 'class' => 'quota-info'));
+
+                $table->add_header(null, self::Q($this->gettext('quotatype')));
+                $table->add_header(null, self::Q($this->gettext('quotatotal')));
+                $table->add_header(null, self::Q($this->gettext('quotaused')));
+
+                foreach ($quota_result['all'] as $root => $data) {
+                    if ($root_cnt > 1 && $root) {
+                        $table->add(array('colspan' => 3, 'class' => 'root'), self::Q($root));
+                    }
+
+                    if ($storage = $data['storage']) {
+                        $percent = min(100, round(($storage['used']/max(1,$storage['total']))*100));
+
+                        $table->add('name', self::Q($this->gettext('quotastorage')));
+                        $table->add(null, $this->show_bytes($storage['total'] * 1024));
+                        $table->add(null, sprintf('%s (%.0f%%)', $this->show_bytes($storage['used'] * 1024), $percent));
+                    }
+                    if ($message = $data['message']) {
+                        $percent = min(100, round(($message['used']/max(1,$message['total']))*100));
+
+                        $table->add('name', self::Q($this->gettext('quotamessage')));
+                        $table->add(null, intval($message['total']));
+                        $table->add(null, sprintf('%d (%.0f%%)', $message['used'], $percent));
+                    }
+                }
+
+                $quota_result['table'] = $table->show();
             }
         }
         else {
@@ -1723,6 +1782,12 @@ class rcmail extends rcube
             $quota_result['percent'] = 0;
         }
 
+        // cleanup
+        unset($quota_result['abort']);
+        if (empty($quota_result['table'])) {
+            unset($quota_result['all']);
+        }
+
         return $quota_result;
     }
 
@@ -1732,8 +1797,9 @@ class rcmail extends rcube
      * @param string $fallback       Fallback message label
      * @param array  $fallback_args  Fallback message label arguments
      * @param string $suffix         Message label suffix
+     * @param array  $params         Additional parameters (type, prefix)
      */
-    public function display_server_error($fallback = null, $fallback_args = null, $suffix = '')
+    public function display_server_error($fallback = null, $fallback_args = null, $suffix = '', $params = array())
     {
         $err_code = $this->storage->get_error_code();
         $res_code = $this->storage->get_response_code();
@@ -1746,7 +1812,7 @@ class rcmail extends rcube
             $error = 'errorreadonly';
         }
         else if ($res_code == rcube_storage::OVERQUOTA) {
-            $error = 'errorroverquota';
+            $error = 'erroroverquota';
         }
         else if ($err_code && ($err_str = $this->storage->get_error_str())) {
             // try to detect access rights problem and display appropriate message
@@ -1754,13 +1820,13 @@ class rcmail extends rcube
                 $error = 'errornoperm';
             }
             // try to detect full mailbox problem and display appropriate message
-            // there can be e.g. "Quota exceeded" or "quotum would exceed"
-            else if (stripos($err_str, 'quot') !== false && stripos($err_str, 'exceed') !== false) {
+            // there can be e.g. "Quota exceeded" / "quotum would exceed" / "Over quota"
+            else if (stripos($err_str, 'quot') !== false && preg_match('/exceed|over/i', $err_str)) {
                 $error = 'erroroverquota';
             }
             else {
                 $error = 'servererrormsg';
-                $args  = array('msg' => $err_str);
+                $args  = array('msg' => rcube::Q($err_str));
             }
         }
         else if ($err_code < 0) {
@@ -1769,13 +1835,21 @@ class rcmail extends rcube
         else if ($fallback) {
             $error = $fallback;
             $args  = $fallback_args;
+            $params['prefix'] = false;
         }
 
         if ($error) {
             if ($suffix && $this->text_exists($error . $suffix)) {
                 $error .= $suffix;
             }
-            $this->output->show_message($error, 'error', $args);
+
+            $msg = $this->gettext(array('name' => $error, 'vars' => $args));
+
+            if ($params['prefix'] && $fallback) {
+                $msg = $this->gettext(array('name' => $fallback, 'vars' => $fallback_args)) . ' ' . $msg;
+            }
+
+            $this->output->show_message($msg, $params['type'] ?: 'error');
         }
     }
 
@@ -1792,21 +1866,25 @@ class rcmail extends rcube
             return;
         }
 
-        $lang = strtolower($_SESSION['language']);
+        $lang_codes = array($_SESSION['language']);
+        $assets_dir = $this->config->get('assets_dir') ?: INSTALL_PATH;
 
-        // TinyMCE uses two-letter lang codes, with exception of Chinese
-        if (strpos($lang, 'zh_') === 0) {
-            $lang = str_replace('_', '-', $lang);
-        }
-        else {
-            $lang = substr($lang, 0, 2);
+        if ($pos = strpos($_SESSION['language'], '_')) {
+            $lang_codes[] = substr($_SESSION['language'], 0, $pos);
         }
 
-        if (!file_exists(INSTALL_PATH . 'program/js/tiny_mce/langs/'.$lang.'.js')) {
+        foreach ($lang_codes as $code) {
+            if (file_exists("$assets_dir/program/js/tinymce/langs/$code.js")) {
+                $lang = $code;
+                break;
+            }
+        }
+
+        if (empty($lang)) {
             $lang = 'en';
         }
 
-        $script = array(
+        $config = array(
             'mode'       => $mode,
             'lang'       => $lang,
             'skin_path'  => $this->output->get_skin_path(),
@@ -1814,9 +1892,11 @@ class rcmail extends rcube
             'spelldict'  => intval($this->config->get('spellcheck_dictionary'))
         );
 
-        $this->output->include_script('tiny_mce/tiny_mce.js');
+        $this->output->add_label('selectimage', 'addimage', 'selectmedia', 'addmedia');
+        $this->output->set_env('editor_config', $config);
+        $this->output->include_css('program/js/tinymce/roundcube/browser.css');
+        $this->output->include_script('tinymce/tinymce.min.js');
         $this->output->include_script('editor.js');
-        $this->output->set_env('html_editor_init', $script);
     }
 
     /**
@@ -1848,8 +1928,9 @@ class rcmail extends rcube
         );
 
         foreach ($emoticons as $idx => $file) {
-            // <img title="Cry" src="http://.../program/js/tiny_mce/plugins/emotions/img/smiley-cry.gif" border="0" alt="Cry" />
-            $search[]  = '/<img title="[a-z ]+" src="https?:\/\/[a-z0-9_.\/-]+\/tiny_mce\/plugins\/emotions\/img\/'.$file.'.gif"[^>]+\/>/i';
+            // <img title="Cry" src="http://.../program/js/tinymce/plugins/emoticons/img/smiley-cry.gif" border="0" alt="Cry" />
+            $file      = preg_quote('program/js/tinymce/plugins/emoticons/img/' . $file . '.gif', '/');
+            $search[]  = '/<img (title="[a-z ]+" )?src="[^"]+' . $file . '"[^>]+\/>/i';
             $replace[] = $idx;
         }
 
@@ -1861,27 +1942,71 @@ class rcmail extends rcube
      */
     public function upload_progress()
     {
-        $prefix = ini_get('apc.rfc1867_prefix');
         $params = array(
             'action' => $this->action,
-            'name' => rcube_utils::get_input_value('_progress', rcube_utils::INPUT_GET),
+            'name'   => rcube_utils::get_input_value('_progress', rcube_utils::INPUT_GET),
         );
 
-        if (function_exists('apc_fetch')) {
-            $status = apc_fetch($prefix . $params['name']);
+        if (function_exists('uploadprogress_get_info')) {
+            $status = uploadprogress_get_info($params['name']);
 
             if (!empty($status)) {
-                $status['percent'] = round($status['current']/$status['total']*100);
-                $params = array_merge($status, $params);
+                $params['current'] = $status['bytes_uploaded'];
+                $params['total']   = $status['bytes_total'];
             }
         }
 
-        if (isset($params['percent']))
-            $params['text'] = $this->gettext(array('name' => 'uploadprogress', 'vars' => array(
-                'percent' => $params['percent'] . '%',
-                'current' => $this->show_bytes($params['current']),
-                'total'   => $this->show_bytes($params['total'])
-        )));
+        if (!isset($status) && filter_var(ini_get('apc.rfc1867'), FILTER_VALIDATE_BOOLEAN)
+            && ini_get('apc.rfc1867_name')
+        ) {
+            $prefix = ini_get('apc.rfc1867_prefix');
+            $status = apc_fetch($prefix . $params['name']);
+
+            if (!empty($status)) {
+                $params['current'] = $status['current'];
+                $params['total']   = $status['total'];
+            }
+        }
+
+        if (!isset($status) && filter_var(ini_get('session.upload_progress.enabled'), FILTER_VALIDATE_BOOLEAN)
+            && ini_get('session.upload_progress.name')
+        ) {
+            $key = ini_get('session.upload_progress.prefix') . $params['name'];
+
+            $params['total']   = $_SESSION[$key]['content_length'];
+            $params['current'] = $_SESSION[$key]['bytes_processed'];
+        }
+
+        if (!empty($params['total'])) {
+            $total = $this->show_bytes($params['total'], $unit);
+            switch ($unit) {
+            case 'GB':
+                $gb      = $params['current']/1073741824;
+                $current = sprintf($gb >= 10 ? "%d" : "%.1f", $gb);
+                break;
+            case 'MB':
+                $mb      = $params['current']/1048576;
+                $current = sprintf($mb >= 10 ? "%d" : "%.1f", $mb);
+                break;
+            case 'KB':
+                $current = round($params['current']/1024);
+                break;
+            case 'B':
+            default:
+                $current = $params['current'];
+                break;
+            }
+
+            $params['percent'] = round($params['current']/$params['total']*100);
+            $params['text']    = $this->gettext(array(
+                'name' => 'uploadprogress',
+                'vars' => array(
+                    'percent' => $params['percent'] . '%',
+                    'current' => $current,
+                    'total'   => $total
+                )
+            ));
+        }
 
         $this->output->command('upload_progress_update', $params);
         $this->output->send();
@@ -1889,13 +2014,24 @@ class rcmail extends rcube
 
     /**
      * Initializes file uploading interface.
+     *
+     * @param $int Optional maximum file size in bytes
      */
-    public function upload_init()
+    public function upload_init($max_size = null)
     {
         // Enable upload progress bar
-        $rfc1867 = filter_var(ini_get('apc.rfc1867'), FILTER_VALIDATE_BOOLEAN);
-        if ($rfc1867 && ($seconds = $this->config->get('upload_progress'))) {
-            if ($field_name = ini_get('apc.rfc1867_name')) {
+        if ($seconds = $this->config->get('upload_progress')) {
+            if (function_exists('uploadprogress_get_info')) {
+                $field_name = 'UPLOAD_IDENTIFIER';
+            }
+            if (!$field_name && filter_var(ini_get('apc.rfc1867'), FILTER_VALIDATE_BOOLEAN)) {
+                $field_name = ini_get('apc.rfc1867_name');
+            }
+            if (!$field_name && filter_var(ini_get('session.upload_progress.enabled'), FILTER_VALIDATE_BOOLEAN)) {
+                $field_name = ini_get('session.upload_progress.name');
+            }
+
+            if ($field_name) {
                 $this->output->set_env('upload_progress_name', $field_name);
                 $this->output->set_env('upload_progress_time', (int) $seconds);
             }
@@ -1909,6 +2045,10 @@ class rcmail extends rcube
             $max_filesize = $max_postsize;
         }
 
+        if ($max_size && $max_size < $max_filesize) {
+            $max_filesize = $max_size;
+        }
+
         $this->output->set_env('max_filesize', $max_filesize);
         $max_filesize = $this->show_bytes($max_filesize);
         $this->output->set_env('filesizeerror', $this->gettext(array(
@@ -1917,6 +2057,76 @@ class rcmail extends rcube
         return $max_filesize;
     }
 
+    /**
+     * Outputs uploaded file content (with image thumbnails support
+     *
+     * @param array $file Upload file data
+     */
+    public function display_uploaded_file($file)
+    {
+        if (empty($file)) {
+            return;
+        }
+
+        $file = $this->plugins->exec_hook('attachment_display', $file);
+
+        if ($file['status']) {
+            if (empty($file['size'])) {
+                $file['size'] = $file['data'] ? strlen($file['data']) : @filesize($file['path']);
+            }
+
+            // generate image thumbnail for file browser in HTML editor
+            if (!empty($_GET['_thumbnail'])) {
+                $temp_dir       = $this->config->get('temp_dir');
+                $thumbnail_size = 80;
+                $mimetype       = $file['mimetype'];
+                $file_ident     = $file['id'] . ':' . $file['mimetype'] . ':' . $file['size'];
+                $cache_basename = $temp_dir . '/' . md5($file_ident . ':' . $this->user->ID . ':' . $thumbnail_size);
+                $cache_file     = $cache_basename . '.thumb';
+
+                // render thumbnail image if not done yet
+                if (!is_file($cache_file)) {
+                    if (!$file['path']) {
+                        $orig_name = $filename = $cache_basename . '.tmp';
+                        file_put_contents($orig_name, $file['data']);
+                    }
+                    else {
+                        $filename = $file['path'];
+                    }
+
+                    $image = new rcube_image($filename);
+                    if ($imgtype = $image->resize($thumbnail_size, $cache_file, true)) {
+                        $mimetype = 'image/' . $imgtype;
+
+                        if ($orig_name) {
+                            unlink($orig_name);
+                        }
+                    }
+                }
+
+                if (is_file($cache_file)) {
+                    // cache for 1h
+                    $this->output->future_expire_header(3600);
+                    header('Content-Type: ' . $mimetype);
+                    header('Content-Length: ' . filesize($cache_file));
+
+                    readfile($cache_file);
+                    exit;
+                }
+            }
+
+            header('Content-Type: ' . $file['mimetype']);
+            header('Content-Length: ' . $file['size']);
+
+            if ($file['data']) {
+                echo $file['data'];
+            }
+            else if ($file['path']) {
+                readfile($file['path']);
+            }
+        }
+    }
+
     /**
      * Initializes client-side autocompletion.
      */
@@ -1978,25 +2188,30 @@ class rcmail extends rcube
     /**
      * Create a human readable string for a number of bytes
      *
-     * @param int Number of bytes
+     * @param int    Number of bytes
+     * @param string Size unit
      *
      * @return string Byte string
      */
-    public function show_bytes($bytes)
+    public function show_bytes($bytes, &$unit = null)
     {
         if ($bytes >= 1073741824) {
-            $gb  = $bytes/1073741824;
-            $str = sprintf($gb>=10 ? "%d " : "%.1f ", $gb) . $this->gettext('GB');
+            $unit = 'GB';
+            $gb   = $bytes/1073741824;
+            $str  = sprintf($gb >= 10 ? "%d " : "%.1f ", $gb) . $this->gettext($unit);
         }
         else if ($bytes >= 1048576) {
-            $mb  = $bytes/1048576;
-            $str = sprintf($mb>=10 ? "%d " : "%.1f ", $mb) . $this->gettext('MB');
+            $unit = 'MB';
+            $mb   = $bytes/1048576;
+            $str  = sprintf($mb >= 10 ? "%d " : "%.1f ", $mb) . $this->gettext($unit);
         }
         else if ($bytes >= 1024) {
-            $str = sprintf("%d ",  round($bytes/1024)) . $this->gettext('KB');
+            $unit = 'KB';
+            $str  = sprintf("%d ",  round($bytes/1024)) . $this->gettext($unit);
         }
         else {
-            $str = sprintf('%d ', $bytes) . $this->gettext('B');
+            $unit = 'B';
+            $str  = sprintf('%d ', $bytes) . $this->gettext($unit);
         }
 
         return $str;
@@ -2026,6 +2241,121 @@ class rcmail extends rcube
         return $size;
     }
 
+    /**
+     * Returns message UID(s) and IMAP folder(s) from GET/POST data
+     *
+     * @param string UID value to decode
+     * @param string Default mailbox value (if not encoded in UIDs)
+     * @param bool   Will be set to True if multi-folder request
+     *
+     * @return array  List of message UIDs per folder
+     */
+    public static function get_uids($uids = null, $mbox = null, &$is_multifolder = false)
+    {
+        // message UID (or comma-separated list of IDs) is provided in
+        // the form of <ID>-<MBOX>[,<ID>-<MBOX>]*
+
+        $_uid  = $uids ?: rcube_utils::get_input_value('_uid', RCUBE_INPUT_GPC);
+        $_mbox = $mbox ?: (string)rcube_utils::get_input_value('_mbox', RCUBE_INPUT_GPC);
+
+        // already a hash array
+        if (is_array($_uid) && !isset($_uid[0])) {
+            return $_uid;
+        }
+
+        $result = array();
+
+        // special case: *
+        if ($_uid == '*' && is_object($_SESSION['search'][1]) && $_SESSION['search'][1]->multi) {
+            $is_multifolder = true;
+            // extract the full list of UIDs per folder from the search set
+            foreach ($_SESSION['search'][1]->sets as $subset) {
+                $mbox = $subset->get_parameters('MAILBOX');
+                $result[$mbox] = $subset->get();
+            }
+        }
+        else {
+            if (is_string($_uid))
+                $_uid = explode(',', $_uid);
+
+            // create a per-folder UIDs array
+            foreach ((array)$_uid as $uid) {
+                list($uid, $mbox) = explode('-', $uid, 2);
+                if (!strlen($mbox)) {
+                    $mbox = $_mbox;
+                }
+                else {
+                    $is_multifolder = true;
+                }
+
+                if ($uid == '*') {
+                    $result[$mbox] = $uid;
+                }
+                else {
+                    $result[$mbox][] = $uid;
+                }
+            }
+        }
+
+        return $result;
+    }
+
+    /**
+     * Get resource file content (with assets_dir support)
+     *
+     * @param string $name File name
+     */
+    public function get_resource_content($name)
+    {
+        if (!strpos($name, '/')) {
+            $name = "program/resources/$name";
+        }
+
+        $assets_dir = $this->config->get('assets_dir');
+
+        if ($assets_dir) {
+            $path = slashify($assets_dir) . $name;
+            if (@file_exists($path)) {
+                $name = $path;
+            }
+        }
+
+        return file_get_contents($name, false);
+    }
+
+    /**
+     * Converts HTML content into plain text
+     *
+     * @param string $html    HTML content
+     * @param array  $options Conversion parameters (width, links, charset)
+     *
+     * @return string Plain text
+     */
+    public function html2text($html, $options = array())
+    {
+        $default_options = array(
+            'links'   => true,
+            'width'   => 75,
+            'body'    => $html,
+            'charset' => RCUBE_CHARSET,
+        );
+
+        $options = array_merge($default_options, (array) $options);
+
+        // Plugins may want to modify HTML in another/additional way
+        $options = $this->plugins->exec_hook('html2text', $options);
+
+        // Convert to text
+        if (!$options['abort']) {
+            $converter = new rcube_html2text($options['body'],
+                false, $options['links'], $options['width'], $options['charset']);
+
+            $options['body'] = rtrim($converter->get_text());
+        }
+
+        return $options['body'];
+    }
+
 
     /************************************************************************
      *********          Deprecated methods (to be removed)          *********
