@@ -64,7 +64,8 @@ class rcube_ldap extends rcube_addressbook
 
     private $base_dn        = '';
     private $groups_base_dn = '';
-    private $group_url;
+    private $group_data;
+    private $group_search_cache;
     private $cache;
 
 
@@ -101,8 +102,8 @@ class rcube_ldap extends rcube_addressbook
             // add group name attrib to the list of attributes to be fetched
             $fetch_attributes[] = $this->prop['groups']['name_attr'];
         }
-        if (is_array($p['group_filters']) && count($p['group_filters'])) {
-            $this->groups = true;
+        if (is_array($p['group_filters'])) {
+            $this->groups = $this->groups || count($p['group_filters']);
 
             foreach ($p['group_filters'] as $k => $group_filter) {
                 // set default name attribute to cn
@@ -233,8 +234,7 @@ class rcube_ldap extends rcube_addressbook
 
         // initialize ldap wrapper object
         $this->ldap = new rcube_ldap_generic($this->prop);
-        $this->ldap->set_cache($this->cache);
-        $this->ldap->set_debug($this->debug);
+        $this->ldap->config_set(array('cache' => $this->cache, 'debug' => $this->debug));
 
         $this->_connect();
     }
@@ -321,8 +321,7 @@ class rcube_ldap extends rcube_addressbook
                             // we need to use a separate LDAP connection
                             if (!empty($this->prop['vlv'])) {
                                 $ldap = new rcube_ldap_generic($this->prop);
-                                $ldap->set_debug($this->debug);
-                                $ldap->set_cache($this->cache);
+                                $ldap->config_set(array('cache' => $this->cache, 'debug' => $this->debug));
                                 if (!$ldap->connect($host)) {
                                     continue;
                                 }
@@ -336,7 +335,7 @@ class rcube_ldap extends rcube_addressbook
                         $res = $ldap->search($search_base_dn, $search_filter, 'sub', $search_attribs);
                         if ($res) {
                             $res->rewind();
-                            $replaces['%dn'] = $res->get_dn();
+                            $replaces['%dn'] = key($res->entries(TRUE));
 
                             // add more replacements from 'search_bind_attrib' config
                             if ($search_bind_attrib) {
@@ -384,7 +383,7 @@ class rcube_ldap extends rcube_addressbook
                         $this->prop['groups'][$k] = strtr($this->prop['groups'][$k], $replaces);
                 }
 
-                if (!empty($this->prop['group_filters'])) {
+                if (is_array($this->prop['group_filters'])) {
                     foreach ($this->prop['group_filters'] as $i => $gf) {
                         if (!empty($gf['base_dn']))
                             $this->prop['group_filters'][$i]['base_dn'] = strtr($gf['base_dn'], $replaces);
@@ -555,30 +554,15 @@ class rcube_ldap extends rcube_addressbook
             $this->result = new rcube_result_set($entries['count'], ($this->list_page-1) * $this->page_size);
         }
         else {
-            $prop    = $this->group_id ? $this->group_data : $this->prop;
-            $base_dn = $this->group_id ? $prop['base_dn'] : $this->base_dn;
-
-            // use global search filter
-            if (!empty($this->filter))
-                $prop['filter'] = $this->filter;
-
             // exec LDAP search if no result resource is stored
-            if ($this->ready && !$this->ldap_result)
-                $this->ldap_result = $this->ldap->search($base_dn, $prop['filter'], $prop['scope'], $this->prop['attributes'], $prop);
+            if ($this->ready && $this->ldap_result === null) {
+                $this->ldap_result = $this->extended_search();
+            }
 
             // count contacts for this user
             $this->result = $this->count();
 
-            // we have a search result resource
-            if ($this->ldap_result && $this->result->count > 0) {
-                // sorting still on the ldap server
-                if ($this->sort_col && $prop['scope'] !== 'base' && !$this->ldap->vlv_active)
-                    $this->ldap_result->sort($this->sort_col);
-
-                // get all entries from the ldap server
-                $entries = $this->ldap_result->entries();
-            }
-
+            $entries = $this->ldap_result;
         }  // end else
 
         // start and end of the page
@@ -589,7 +573,8 @@ class rcube_ldap extends rcube_addressbook
 
         // filter entries for this page
         for ($i = $start_row; $i < min($entries['count'], $last_row); $i++)
-            $this->result->add($this->_ldap2result($entries[$i]));
+            if ($entries[$i])
+                $this->result->add($this->_ldap2result($entries[$i]));
 
         return $this->result;
     }
@@ -699,8 +684,9 @@ class rcube_ldap extends rcube_addressbook
 
         for ($i=0; $i < $entry['memberurl']['count']; $i++) {
             // extract components from url
-            if (!preg_match('!ldap:///([^\?]+)\?\?(\w+)\?(.*)$!', $entry['memberurl'][$i], $m))
+            if (!preg_match('!ldap://[^/]*/([^\?]+)\?\?(\w+)\?(.*)$!', $entry['memberurl'][$i], $m)) {
                 continue;
+            }
 
             // add search filter if any
             $filter = $this->filter ? '(&(' . $m[3] . ')(' . $this->filter . '))' : $m[3];
@@ -733,15 +719,12 @@ class rcube_ldap extends rcube_addressbook
      *
      * @param mixed   $fields   The field name of array of field names to search in
      * @param mixed   $value    Search value (or array of values when $fields is array)
-     * @param int     $mode     Matching mode:
-     *                          0 - partial (*abc*),
-     *                          1 - strict (=),
-     *                          2 - prefix (abc*)
+     * @param int     $mode     Matching mode. Sum of rcube_addressbook::SEARCH_*
      * @param boolean $select   True if results are requested, False if count only
      * @param boolean $nocount  (Not used)
      * @param array   $required List of fields that cannot be empty
      *
-     * @return array  Indexed list of contact records and 'count' value
+     * @return rcube_result_set List of contact records
      */
     function search($fields, $value, $mode=0, $select=true, $nocount=false, $required=array())
     {
@@ -767,16 +750,16 @@ class rcube_ldap extends rcube_addressbook
         if ($this->prop['vlv_search'] && $this->ready && join(',', (array)$fields) == join(',', $list_fields)) {
             $this->result = new rcube_result_set(0);
 
-            $search_suffix = $this->prop['fuzzy_search'] && $mode != 1 ? '*' : '';
+            $this->ldap->config_set('fuzzy_search', intval($this->prop['fuzzy_search'] && !($mode & rcube_addressbook::SEARCH_STRICT)));
             $ldap_data = $this->ldap->search($this->base_dn, $this->prop['filter'], $this->prop['scope'], $this->prop['attributes'],
-                array('search' => $value . $search_suffix /*, 'sort' => $this->prop['sort'] */));
+                array('search' => $value /*, 'sort' => $this->prop['sort'] */));
             if ($ldap_data === false) {
                 return $this->result;
             }
 
             // get all entries of this page and post-filter those that really match the query
             $search = mb_strtolower($value);
-            foreach ($ldap_data as $i => $entry) {
+            foreach ($ldap_data as $entry) {
                 $rec = $this->_ldap2result($entry);
                 foreach ($fields as $f) {
                     foreach ((array)$rec[$f] as $val) {
@@ -792,33 +775,24 @@ class rcube_ldap extends rcube_addressbook
             return $this->result;
         }
 
-        // use AND operator for advanced searches
-        $filter = is_array($value) ? '(&' : '(|';
-        // set wildcards
-        $wp = $ws = '';
-        if (!empty($this->prop['fuzzy_search']) && $mode != 1) {
-            $ws = '*';
-            if (!$mode) {
-                $wp = '*';
-            }
-        }
+        // advanced per-attribute search
+        if (is_array($value)) {
+            // use AND operator for advanced searches
+            $filter = '(&';
 
-        if ($fields == '*') {
-            // search_fields are required for fulltext search
-            if (empty($this->prop['search_fields'])) {
-                $this->set_error(self::ERROR_SEARCH, 'nofulltextsearch');
-                $this->result = new rcube_result_set();
-                return $this->result;
-            }
-            if (is_array($this->prop['search_fields'])) {
-                foreach ($this->prop['search_fields'] as $field) {
-                    $filter .= "($field=$wp" . rcube_ldap_generic::quote_string($value) . "$ws)";
+            // set wildcards
+            $wp = $ws = '';
+            if (!empty($this->prop['fuzzy_search']) && !($mode & rcube_addressbook::SEARCH_STRICT)) {
+                $ws = '*';
+                if (!($mode & rcube_addressbook::SEARCH_PREFIX)) {
+                    $wp = '*';
                 }
             }
-        }
-        else {
+
             foreach ((array)$fields as $idx => $field) {
-                $val = is_array($value) ? $value[$idx] : $value;
+                $val = $value[$idx];
+                if (!strlen($val))
+                    continue;
                 if ($attrs = $this->_map_field($field)) {
                     if (count($attrs) > 1)
                         $filter .= '(|';
@@ -828,8 +802,32 @@ class rcube_ldap extends rcube_addressbook
                         $filter .= ')';
                 }
             }
+
+            $filter .= ')';
+        }
+        else {
+            if ($fields == '*') {
+                // search_fields are required for fulltext search
+                if (empty($this->prop['search_fields'])) {
+                    $this->set_error(self::ERROR_SEARCH, 'nofulltextsearch');
+                    $this->result = new rcube_result_set();
+                    return $this->result;
+                }
+                $attributes = (array)$this->prop['search_fields'];
+            }
+            else {
+                // map address book fields into ldap attributes
+                $attributes = array();
+                foreach ((array) $fields as $field) {
+                    if ($this->coltypes[$field] && ($attrs = $this->coltypes[$field]['attributes'])) {
+                        $attributes = array_merge($attributes, (array) $attrs);
+                    }
+                }
+            }
+
+            // compose a full-text-like search filter
+            $filter = rcube_ldap_generic::fulltext_search_filter($value, $attributes, $mode);
         }
-        $filter .= ')';
 
         // add required (non empty) fields filter
         $req_filter = '';
@@ -852,9 +850,9 @@ class rcube_ldap extends rcube_addressbook
         // avoid double-wildcard if $value is empty
         $filter = preg_replace('/\*+/', '*', $filter);
 
-        // add general filter to query
-        if (!empty($this->prop['filter']))
-            $filter = '(&(' . preg_replace('/^\(|\)$/', '', $this->prop['filter']) . ')' . $filter . ')';
+        if ($mode & rcube_addressbook::SEARCH_GROUPS) {
+            $filter = 'e:' . $filter;
+        }
 
         // set filter string and execute search
         $this->set_search_set($filter);
@@ -876,26 +874,101 @@ class rcube_ldap extends rcube_addressbook
     function count()
     {
         $count = 0;
-        if ($this->ldap_result) {
-            $count = $this->ldap_result->count();
+        if (!empty($this->ldap_result)) {
+            $count = $this->ldap_result['count'];
         }
         else if ($this->group_id && $this->group_data['dn']) {
             $count = count($this->list_group_members($this->group_data['dn'], true));
         }
         // We have a connection but no result set, attempt to get one.
         else if ($this->ready) {
-            $prop    = $this->group_id ? $this->group_data : $this->prop;
-            $base_dn = $this->group_id ? $this->group_base_dn : $this->base_dn;
-
-            if (!empty($this->filter)) {  // Use global search filter
-                $prop['filter'] = $this->filter;
-            }
-            $count = $this->ldap->search($base_dn, $prop['filter'], $prop['scope'], array('dn'), $prop, true);
+            $count = $this->extended_search(true);
         }
 
         return new rcube_result_set($count, ($this->list_page-1) * $this->page_size);
     }
 
+    /**
+     * Wrapper on LDAP searches with group_filters support, which
+     * allows searching for contacts AND groups.
+     *
+     * @param bool $count Return count instead of the records
+     *
+     * @return int|array Count of records or the result array (with 'count' item)
+     */
+    protected function extended_search($count = false)
+    {
+        $prop    = $this->group_id ? $this->group_data : $this->prop;
+        $base_dn = $this->group_id ? $this->groups_base_dn : $this->base_dn;
+        $attrs   = $count ? array('dn') : $this->prop['attributes'];
+        $entries = array();
+
+        // Use global search filter
+        if ($filter = $this->filter) {
+            if ($filter[0] == 'e' && $filter[1] == ':') {
+                $filter = substr($filter, 2);
+                $is_extended_search = !$this->group_id;
+            }
+
+            $prop['filter'] = $filter;
+
+            // add general filter to query
+            if (!empty($this->prop['filter'])) {
+                $prop['filter'] = '(&(' . preg_replace('/^\(|\)$/', '', $this->prop['filter']) . ')' . $prop['filter'] . ')';
+            }
+        }
+
+        $result = $this->ldap->search($base_dn, $prop['filter'], $prop['scope'], $attrs, $prop, $count);
+
+        // we have a search result resource, get all entries
+        if (!$count && $result) {
+            $result_count = $result->count();
+            $result       = $result->entries();
+            unset($result['count']);
+        }
+
+        // search for groups
+        if ($is_extended_search
+            && is_array($this->prop['group_filters'])
+            && !empty($this->prop['groups']['filter'])
+        ) {
+            $filter = '(&(' . preg_replace('/^\(|\)$/', '', $this->prop['groups']['filter']) . ')' . $filter . ')';
+
+            // for groups we may use cn instead of displayname...
+            if ($this->prop['fieldmap']['name'] != $this->prop['groups']['name_attr']) {
+                $filter = str_replace(strtolower($this->prop['fieldmap']['name']) . '=', $this->prop['groups']['name_attr'] . '=', $filter);
+            }
+
+            $name_attr  = $this->prop['groups']['name_attr'];
+            $email_attr = $this->prop['groups']['email_attr'] ?: 'mail';
+            $attrs      = array_unique(array('dn', 'objectClass', $name_attr, $email_attr));
+
+            $res = $this->ldap->search($this->groups_base_dn, $filter, $this->prop['groups']['scope'], $attrs, $prop, $count);
+
+            if ($count && $res) {
+                $result += $res;
+            }
+            else if (!$count && $res && ($res_count = $res->count())) {
+                $res = $res->entries();
+                unset($res['count']);
+                $result = array_merge($result, $res);
+                $result_count += $res_count;
+            }
+        }
+
+        if (!$count && $result) {
+            // sorting
+            if ($this->sort_col && $prop['scope'] !== 'base' && !$this->ldap->vlv_active) {
+                usort($result, array($this, '_entry_sort_cmp'));
+            }
+
+            $result['count'] = $result_count;
+            $this->result_entries = $result;
+        }
+
+        return $result;
+    }
+
 
     /**
      * Return the last result set
@@ -907,7 +980,6 @@ class rcube_ldap extends rcube_addressbook
         return $this->result;
     }
 
-
     /**
      * Get a specific contact record
      *
@@ -949,6 +1021,23 @@ class rcube_ldap extends rcube_addressbook
         return $assoc ? $res : $this->result;
     }
 
+    /**
+     * Returns the last error occurred (e.g. when updating/inserting failed)
+     *
+     * @return array Hash array with the following fields: type, message
+     */
+    function get_error()
+    {
+        $err = $this->error;
+
+        // check ldap connection for errors
+        if (!$err && $this->ldap->get_error()) {
+            $err = array(self::ERROR_SEARCH, $this->ldap->get_error());
+        }
+
+        return $err;
+    }
+
 
     /**
      * Check the given data before saving.
@@ -1067,7 +1156,7 @@ class rcube_ldap extends rcube_addressbook
             }
         }
 
-        if (!$this->ldap->add($dn, $newentry)) {
+        if (!$this->ldap->add_entry($dn, $newentry)) {
             $this->set_error(self::ERROR_SAVING, 'errorsaving');
             return false;
         }
@@ -1079,7 +1168,7 @@ class rcube_ldap extends rcube_addressbook
                 'objectClass' => (array) $this->prop['sub_fields'][$xidx],
             );
 
-            $this->ldap->add($xdn, $xf);
+            $this->ldap->add_entry($xdn, $xf);
         }
 
         $dn = self::dn_encode($dn);
@@ -1222,7 +1311,7 @@ class rcube_ldap extends rcube_addressbook
         if (!empty($subdeldata)) {
             foreach ($subdeldata as $fld => $val) {
                 $subdn = $fld.'='.rcube_ldap_generic::quote_string($val).','.$dn;
-                if (!$this->ldap->delete($subdn)) {
+                if (!$this->ldap->delete_entry($subdn)) {
                     return false;
                 }
             }
@@ -1266,7 +1355,7 @@ class rcube_ldap extends rcube_addressbook
                     $fld => $val,
                     'objectClass' => (array) $this->prop['sub_fields'][$fld],
                 );
-                $this->ldap->add($subdn, $xf);
+                $this->ldap->add_entry($subdn, $xf);
             }
         }
 
@@ -1296,7 +1385,7 @@ class rcube_ldap extends rcube_addressbook
             if ($this->sub_filter) {
                 if ($entries = $this->ldap->list_entries($dn, $this->sub_filter)) {
                     foreach ($entries as $entry) {
-                        if (!$this->ldap->delete($entry['dn'])) {
+                        if (!$this->ldap->delete_entry($entry['dn'])) {
                             $this->set_error(self::ERROR_SAVING, 'errorsaving');
                             return false;
                         }
@@ -1305,12 +1394,12 @@ class rcube_ldap extends rcube_addressbook
             }
 
             // Delete the record.
-            if (!$this->ldap->delete($dn)) {
+            if (!$this->ldap->delete_entry($dn)) {
                 $this->set_error(self::ERROR_SAVING, 'errorsaving');
                 return false;
             }
 
-            // remove contact from all groups where he was member
+            // remove contact from all groups where he was a member
             if ($this->groups) {
                 $dn = self::dn_encode($dn);
                 $group_ids = $this->get_record_groups($dn);
@@ -1343,7 +1432,7 @@ class rcube_ldap extends rcube_addressbook
 
         if ($with_groups && $this->groups && ($groups = $this->_fetch_groups()) && count($groups)) {
             foreach ($groups as $group) {
-                $this->ldap->delete($group['dn']);
+                $this->ldap->delete_entry($group['dn']);
             }
 
             if ($this->cache) {
@@ -1411,6 +1500,16 @@ class rcube_ldap extends rcube_addressbook
             $fieldmap['name'] = $this->group_data['name_attr'] ? $this->group_data['name_attr'] : $this->prop['groups']['name_attr'];
         }
 
+        // assign object type from object class mapping
+        if (!empty($this->prop['class_type_map'])) {
+            foreach (array_map('strtolower', (array)$rec['objectclass']) as $objcls) {
+                if (!empty($this->prop['class_type_map'][$objcls])) {
+                    $out['_type'] = $this->prop['class_type_map'][$objcls];
+                    break;
+                }
+            }
+        }
+
         foreach ($fieldmap as $rf => $lf)
         {
             for ($i=0; $i < $rec[$lf]['count']; $i++) {
@@ -1506,7 +1605,6 @@ class rcube_ldap extends rcube_addressbook
         return $ldap_data;
     }
 
-
     /**
      * Returns unified attribute name (resolving aliases)
      */
@@ -1537,17 +1635,6 @@ class rcube_ldap extends rcube_addressbook
         return count(array_intersect(array_keys($this->group_types), $classes)) > 0;
     }
 
-    /**
-     * Prints debug info to the log
-     */
-    private function _debug($str)
-    {
-        if ($this->debug) {
-            rcube::write_log('ldap', $str);
-        }
-    }
-
-
     /**
      * Activate/deactivate debug mode
      *
@@ -1558,11 +1645,10 @@ class rcube_ldap extends rcube_addressbook
         $this->debug = $dbg;
 
         if ($this->ldap) {
-            $this->ldap->set_debug($dbg);
+            $this->ldap->config_set('debug', $dbg);
         }
     }
 
-
     /**
      * Setter for the current group
      */
@@ -1582,10 +1668,7 @@ class rcube_ldap extends rcube_addressbook
      * List all active contact groups of this source
      *
      * @param string  Optional search string to match group name
-     * @param int     Matching mode:
-     *                0 - partial (*abc*),
-     *                1 - strict (=),
-     *                2 - prefix (abc*)
+     * @param int     Matching mode. Sum of rcube_addressbook::SEARCH_*
      *
      * @return array  Indexed list of contact groups, each a hash array
      */
@@ -1595,12 +1678,12 @@ class rcube_ldap extends rcube_addressbook
             return array();
         }
 
-        $group_cache = $this->_fetch_groups();
+        $group_cache = $this->_fetch_groups($search, $mode);
         $groups      = array();
 
         if ($search) {
             foreach ($group_cache as $group) {
-                if ($this->compare_search_value('name', $group['name'], $search, $mode)) {
+                if ($this->compare_search_value('name', $group['name'], mb_strtolower($search), $mode)) {
                     $groups[] = $group;
                 }
             }
@@ -1615,10 +1698,19 @@ class rcube_ldap extends rcube_addressbook
     /**
      * Fetch groups from server
      */
-    private function _fetch_groups($vlv_page = null)
+    private function _fetch_groups($search = null, $mode = 0, $vlv_page = null)
     {
+        // reset group search cache
+        if ($search !== null && $vlv_page === null) {
+            $this->group_search_cache = null;
+        }
+        // return in-memory cache from previous search results
+        else if (is_array($this->group_search_cache) && $vlv_page === null) {
+            return $this->group_search_cache;
+        }
+
         // special case: list groups from 'group_filters' config
-        if ($vlv_page === null && !empty($this->prop['group_filters'])) {
+        if ($vlv_page === null && $search === null && is_array($this->prop['group_filters'])) {
             $groups = array();
             $rcube  = rcube::get_instance();
 
@@ -1635,7 +1727,7 @@ class rcube_ldap extends rcube_addressbook
             return $groups;
         }
 
-        if ($this->cache && $vlv_page === null && ($groups = $this->cache->get('groups')) !== null) {
+        if ($this->cache && $search === null && $vlv_page === null && ($groups = $this->cache->get('groups')) !== null) {
             return $groups;
         }
 
@@ -1657,12 +1749,28 @@ class rcube_ldap extends rcube_addressbook
             }
 
             $ldap = clone $this->ldap;
-            $ldap->set_config($this->prop['groups']);
+            $ldap->config_set($this->prop['groups']);
             $ldap->set_vlv_page($vlv_page+1, $page_size);
         }
 
-        $attrs     = array_unique(array('dn', 'objectClass', $name_attr, $email_attr, $sort_attr));
-        $ldap_data = $ldap->search($base_dn, $filter, $scope, $attrs, $this->prop['groups']);
+        $props = array('sort' => $this->prop['groups']['sort']);
+        $attrs = array_unique(array('dn', 'objectClass', $name_attr, $email_attr, $sort_attr));
+
+        // add search filter
+        if ($search !== null) {
+            // set wildcards
+            $wp = $ws = '';
+            if (!empty($this->prop['fuzzy_search']) && !($mode & rcube_addressbook::SEARCH_STRICT)) {
+                $ws = '*';
+                if (!($mode & rcube_addressbook::SEARCH_PREFIX)) {
+                    $wp = '*';
+                }
+            }
+            $filter = "(&$filter($name_attr=$wp" . rcube_ldap_generic::quote_string($search) . "$ws))";
+            $props['search'] = $wp . $search . $ws;
+        }
+
+        $ldap_data = $ldap->search($base_dn, $filter, $scope, $attrs, $props);
 
         if ($ldap_data === false) {
             return array();
@@ -1699,7 +1807,7 @@ class rcube_ldap extends rcube_addressbook
 
         // call recursively until we have fetched all groups
         while ($this->prop['groups']['vlv'] && $group_count == $page_size) {
-            $next_page   = $this->_fetch_groups(++$vlv_page);
+            $next_page   = $this->_fetch_groups($search, $mode, ++$vlv_page);
             $groups      = array_merge($groups, $next_page);
             $group_count = count($next_page);
         }
@@ -1710,9 +1818,12 @@ class rcube_ldap extends rcube_addressbook
         }
 
         // cache this
-        if ($this->cache) {
+        if ($this->cache && $search === null) {
             $this->cache->set('groups', $groups);
         }
+        else if ($search !== null) {
+            $this->group_search_cache = $groups;
+        }
 
         return $groups;
     }
@@ -1781,7 +1892,7 @@ class rcube_ldap extends rcube_addressbook
             $member_attr => '',
         );
 
-        if (!$this->ldap->add($new_dn, $new_entry)) {
+        if (!$this->ldap->add_entry($new_dn, $new_entry)) {
             $this->set_error(self::ERROR_SAVING, 'errorsaving');
             return false;
         }
@@ -1804,7 +1915,7 @@ class rcube_ldap extends rcube_addressbook
         $group_cache = $this->_fetch_groups();
         $del_dn      = $group_cache[$group_id]['dn'];
 
-        if (!$this->ldap->delete($del_dn)) {
+        if (!$this->ldap->delete_entry($del_dn)) {
             $this->set_error(self::ERROR_SAVING, 'errorsaving');
             return false;
         }
@@ -1939,7 +2050,7 @@ class rcube_ldap extends rcube_addressbook
         $filter = strtr("(|(member=$contact_dn)(uniqueMember=$contact_dn)$add_filter)", array('\\' => '\\\\'));
 
         $ldap_data = $this->ldap->search($base_dn, $filter, 'sub', array('dn', $name_attr));
-        if ($res === false) {
+        if ($ldap_data === false) {
             return array();
         }
 
